/***************************************************************************/
/*
    Gary Forest
    gary@csd4.csd.uwm.edu
    100 S. Elmridge
    Brookfield, WI 53005

    cvip.c - Configurable Visual Interference Pattern

    This program is distributed as shareware.  You are free to improve on
    and distribute this software.  All I ask is to leave the header with
    my name intact and to comment any changes made.  Any comments or
    suggestions are welcome.

    If you use and enjoy the program contributions are gladly accepted, they
    encourage the future creation, enhancement and distribution of programs.

    Enhancements in the works:
      - n-ball version, possibly with keys to increase, decrease number
	of balls on screen at given time.
      - perform spline between balls in n-ball version to give smooth
	curve instead of n-gon.
      - possibly make it a TSR/screen blanker program.

    The program is written with Microsoft C version 5.1

    Cvip generates Visual Interference Patterns, also know as Moire Patterns.

    The program works by plotting the path taken by two balls enclosed
    within the screen boundaries and linking the positions of the balls
    with a line.  The balls encounter perfectly elastic collisions
    with the boundaries.

    The program allows various parameters to be configured:
	      - Rate of color change
	      - Buffer length
	      - Speed of Moire
	      - Increment taken by each ball



    The keys linked to each of these functions are as follows:


	  Rate of
	  Color     Moire    Buffer
	  Change    Speed    Length

(INCREASE)  7         8         9 --|
				    |-- All keys on Numeric Keypad
(DECREASE)  1         2         3 --|


	 F1 - Increase increment of first ball
	 F2 - Decrease increment of first ball
	 F3 - Increase increment of second ball
	 F4 - Decrease increment of second ball


    The program should work on all video configurations better than CGA.
    It will attempt to use the best resolution avaliable and configure
    itself to utilize it.  Currently it recognizes through VGA.
*/
/***************************************************************************/

#include <stdio.h>
#include <graph.h>
#include <malloc.h>
#include <dos.h>
#include <bios.h>
#include <stdlib.h>

#define DEFAULT_BUFF_LEN 50
#define MAX_BUFF_LEN 500
#define MAX_CLR_CHG_INT 32767
#define MAX_SLEEP_TIME 327

struct line_ends
    {int xpos1;
     int xpos2;
     int ypos1;
     int ypos2;
    };

struct line_ends line_buffer[MAX_BUFF_LEN];

int set_mode (void);
struct videoconfig vc;
char error_message [] = "This program requires a graphics interface";

main()
    {
    int xpos1,xpos2,ypos1,ypos2;      /* current position of balls */
    int slope1,slope2;                /* current slope of each ball path */
    int start1,start2;                /* y intercept of each ball path */
    int incr1,incr2;                  /* x increment of each ball */
    int buff_ptr;                     /* next postion in line buffer */
    int current_color;                /* current draw color */
    int c,sleep_time;                 /* delay time between draws */
    int percent_sleep_time;           /* sleep_time = MAX_SLEEP_TIME * percent_sleep_time */
    int run_flag = 1;                 /* continue program flag */
    unsigned key_pressed;             /* ascii value & scan code of key pressed */
    int buff_size;                    /* current size of circular buffer */
    int buff_size_changed = 0;        /* flag to indicate buffer size changed */
    int old_buff_size;                /* buffer size before change */
    int color_change_int;             /* number of lines drawn before color change */
    int num_this_color = 0;           /* number of lines drawn this color */
    struct dostime_t current_time;    /* current time (to seed random num generator) */
    int get_rand();                   /* returns random integer */
    int set_best_graph_mode();        /* attempts to set best graphics mode */

     if (!set_best_graph_mode())
	 {
	 _setvideomode(_DEFAULTMODE);
	 fprintf (stderr,"%s\n", error_message);
	 exit(-1);
	 }
     _getvideoconfig(&vc);            /* gets num pixels & num colors */


    _dos_gettime(&current_time);
    srand(current_time.hsecond);     /* seed random number gen with current */
				     /* hundredth of seconds */
     start1 = get_rand(vc.numypixels);
     incr1 = get_rand(2)+1;
     xpos1 = 0;
     slope1 = get_rand(3);
     start2 = get_rand(vc.numypixels);
     incr2 = get_rand(2)+1;
     xpos2 = 0;
     slope2 =-1*get_rand(3);
     current_color = get_rand(vc.numcolors);
     color_change_int = get_rand(50);
     percent_sleep_time = get_rand(50);
     sleep_time = 327 * percent_sleep_time;
     buff_size = get_rand(DEFAULT_BUFF_LEN) + 5;

     for(buff_ptr=0;buff_ptr<MAX_BUFF_LEN;++buff_ptr)
	 {
	 line_buffer[buff_ptr].xpos1 = 0;
	 line_buffer[buff_ptr].xpos2 = 0;
	 line_buffer[buff_ptr].ypos1 = 0;
	 line_buffer[buff_ptr].ypos2 = 0;
	 }

     buff_ptr = 0;

     while(run_flag)
	 {
	 if(kbhit())            /* if a key is pressed */
	     {
				/* get key pressed, lower 8 bits contains ascii value */
				/* of key, upper 8 bits scan code */
	     key_pressed = _bios_keybrd(_KEYBRD_READ);

				/* get upper 8 bits, scan code */
	     key_pressed = key_pressed>>8;

	     switch (key_pressed) {
						    /* 2 from num key pad */
						    /* decrease speed */
		 case 0x50: if(percent_sleep_time<100)
				{
				percent_sleep_time = percent_sleep_time + 1;
				sleep_time = MAX_SLEEP_TIME * percent_sleep_time;
				}
			    else
				printf("%c",7);
			    break;

						   /* 8 from num key pad */
						   /* increase speed */
		 case 0x48: if(percent_sleep_time>0)
				{
				percent_sleep_time = percent_sleep_time - 1;
				sleep_time = MAX_SLEEP_TIME * percent_sleep_time;
				}
			    else
				printf("%c",7);

			    break;

						  /* 9 from num key pad */
						  /* increase buffer length */
		 case 0x49: if(buff_size<MAX_BUFF_LEN) ++buff_size; else printf("%c",7);
			    break;

						  /* 3 from num key pad */
						  /* decrease buffer length */
		 case 0x51: if(buff_size>0)
				{
				if(!buff_size_changed)
				    old_buff_size = buff_size;
				buff_size_changed = 1;
				--buff_size;
				}
			    else
				printf("%c",7);
			    break;

						 /* 1 from num key pad */
						 /* decrease rate of color change */
		 case 0x4f: if(color_change_int<MAX_CLR_CHG_INT) ++color_change_int; else printf("%c",7);
			    break;

						 /* 7 from num key pad */
						 /* increase rate of color change */
		 case 0x47: if(color_change_int>0) --color_change_int; else printf("%c",7);
			    break;

						 /* F1 increase ball 1 increment */
		 case 0x3b: if(incr1>0) ++incr1; else --incr1;
			    break;

						 /* F2 decrease ball 1 increment */
		 case 0x3c: if(incr1>0) --incr1; else ++incr1;
			    break;

						 /* F3 increase ball 2 increment */
		 case 0x3d: if(incr2>0) ++incr2; else --incr2;
			    break;

						 /* F4 decrease ball 2 increment */
		 case 0x3e: if(incr2>0) --incr2; else ++incr2;
			    break;

						 /* Any other key, end program */
		 default :  run_flag = 0;
			    break;
		 }
	     }

	     /* needed short sleep interval, I know that this is machine dependant */
	     /* but any sleep routine was too slow */
	 for(c=0;c<sleep_time;++c);

	 xpos1 = xpos1 + incr1;

				/* if ball is at either boundary, turn around */
	 if (xpos1 > vc.numxpixels || xpos1 < 0)
	     {
	     xpos1 = xpos1 - incr1;
	     slope1 = -slope1;
	     start1 = ypos1 - slope1 * xpos1;
	     xpos1 = xpos1 - incr1;
	     incr1 = -incr1;
	     }
	 ypos1 = slope1 * xpos1 + start1;
	 if (ypos1 > vc.numypixels || ypos1 < 0)
	     {
	     slope1 = -slope1;
	     start1 = ypos1 - slope1 * xpos1;
	     ypos1 = slope1 * xpos1 + start1;
	     }


	 xpos2 = xpos2 + incr2;
	 if (xpos2 > vc.numxpixels || xpos2 < 0)
	     {
	     xpos2 = xpos2 - incr2;
	     slope2 = -slope2;
	     start2 = ypos2 - slope2 * xpos2;
	     xpos2 = xpos2 - incr2;
	     incr2 = -incr2;
	     }
	 ypos2 = slope2 * xpos2 + start2;
	 if (ypos2 > vc.numypixels || ypos2 < 0)
	     {
	     slope2 = -slope2;
	     start2 = ypos2 - slope2 * xpos2;
	     ypos2 = slope2 * xpos2 + start2;
	     }

			/* store the current line in buffer so it can be erased later */
	 line_buffer[buff_ptr].xpos1 = xpos1;
	 line_buffer[buff_ptr].xpos2 = xpos2;
	 line_buffer[buff_ptr].ypos1 = ypos1;
	 line_buffer[buff_ptr].ypos2 = ypos2;

			/* do we have to change colors yet? */
	 if(num_this_color>color_change_int)
	     {
	     current_color = get_rand(vc.numcolors);
	     num_this_color = 0;
	     }

			/* draw this line */
	 _setcolor(current_color);
	 _moveto(line_buffer[buff_ptr].xpos1,line_buffer[buff_ptr].ypos1);
	 _lineto(line_buffer[buff_ptr].xpos2,line_buffer[buff_ptr].ypos2);
	 ++num_this_color;
	 ++buff_ptr;

			/* if we are at the begining of the buffer reset */
			/* pointer to begining.  If buffer size has changed */
			/* erase any lines left at the end of buffer */
	 if(buff_ptr >= buff_size)
	     {
	     if(buff_size_changed)
		 {
		 _setcolor(0);
		 for(c=buff_ptr;c<=old_buff_size;++c)
		     {
		     _moveto(line_buffer[c].xpos1,line_buffer[c].ypos1);
		     _lineto(line_buffer[c].xpos2,line_buffer[c].ypos2);
		     }
		 buff_size_changed = 0;
		 }
	     buff_ptr = 0;
	     }

			/* erase next line in buffer */
	 _setcolor(0);
	 _moveto(line_buffer[buff_ptr].xpos1,line_buffer[buff_ptr].ypos1);
	 _lineto(line_buffer[buff_ptr].xpos2,line_buffer[buff_ptr].ypos2);

	 }

	    /* done with program, return to default video mode */
   _setvideomode (_DEFAULTMODE);
}

/******************************************************/
/*  return random number between 1 and base passed in */
/******************************************************/
int get_rand(base)
int base;
    {
    return((int)((float)base*(float)rand()/32767.0)+1.0);
    }

/********************************************************/
/* attempt to set graphics video mode, return 0 on fail */
/********************************************************/
int set_best_graph_mode()
    {
    int c;

    if(_setvideomode(_VRES16COLOR))
	return(_VRES16COLOR);
    if(_setvideomode(_ERESCOLOR))
	return(_ERESCOLOR);
    if(_setvideomode(_MRES4COLOR))
	return(_MRES4COLOR);
    else
	return(0);
    }

